/*
 kwc.c

 KWC -- Key Width Calculator -- Main

 Copyright (c) 2000 Edward V. POPKOV
 Distributed under GPL, see COPYING for details

 "Edward V. POPKOV" <evpopkov@carry.netnet.lv>

 $Id: kwc.c,v 0.2 2000/07/20 11:30:35 root Exp root $

 $Log: kwc.c,v $
 Revision 0.2  2000/07/20 11:30:35  root
 Floating overflow checks

 Revision 0.1  2000/07/15 03:41:07  root
 Moved math to main(), changed coverage calc. and output format

 Revision 0.0  2000/07/09 21:42:22  root
 Initial check-in

*/

#ifndef _KWC_
#define _KWC_

#include <kwc.h>

#define KWC_ARGC_MIN	2
#define KWC_USAGE	"Usage: kwc <val0> [<val1> [ ... <valn>]], where\n\
\t<valx>\tvalue(s) to be multiplied to make the number of combinations.\n\n"
#define KWC_KEY_MSG	"\nKey must cover %.0f values.\n\
Key width is %.0f bits (%.0f bytes of %d bits).\n"
#define KWC_CVR_MSG	"Key coverage:\n\tactual values:\t%10.0f\n\
\tdifference:\t%10.0f\n\
\tratio:\t\t%10.2f%%\n\n"

void kwc_errexit( const char * )
	__attribute__(( __regparm__( 1 ), __noreturn__ ));

void kwc_errexit( const char * msg )
{
	kwc_err( msg );
	exit( EXIT_FAILURE );

}/* kwc_errexit */

int main( int argc, char * argv[])
{
	if ( argc < KWC_ARGC_MIN ) {
		printf( KWC_USAGE );
	} else {

		int i;
		kwc_arg_t l;
		kwc_res_t a;
		kwc_res_t b;
		kwc_res_t c;

		a = 1.00;
		for ( i = 1; i < argc; i++ ) {
			if ( ! kwc_char_check( argv[ i ]))
				kwc_errexit( "Invalid command line option" );
			if ( ! kwc_char_tol( &l, argv[ i ]))
				kwc_errexit( "Error converting to numeric" );
			if ( l <= KWC_EMPTY )
				kwc_errexit( "Negative or zero miltiplier" );
			if ( l > 1 )
				a *= (kwc_res_t) l;
			if ( isinf( a ))
				kwc_errexit( "Floating overflow" );
		}

		b = ceil( log10( a ) / log10( 2 ));
		c = ceil( b / (kwc_res_t) CHARBITS );
		printf( KWC_KEY_MSG, a, b, c, CHARBITS );

		b = 1.00;
		c *= CHARBITS;
		while ( c >= (kwc_res_t) LONGBITS ) {
			b *= 2.00;
			if ( isinf( b ))
				kwc_errexit( "Floating overflow" );
			c--;
		}
		b *= (kwc_res_t) ( 1UL << (unsigned long) c );
		if ( isinf( b ))
			kwc_errexit( "Floating overflow" );
		printf( KWC_CVR_MSG, b, b - a, a / b * 100.00 );

	}

	return EXIT_SUCCESS;

}/* main */

#endif /*_KWC_*/

/* End of kwc.c */
